part of 'auth_confirm_bloc.dart';

sealed class AuthConfirmState extends Equatable {
  const AuthConfirmState(this.isTimerVisible);

  final bool isTimerVisible;
}

class InitialState extends AuthConfirmState {
  const InitialState(super.isTimerVisible);

  @override
  List<Object?> get props => [super.isTimerVisible];
}

class LoadingState extends AuthConfirmState {
  const LoadingState(super.isTimerVisible);

  @override
  List<Object?> get props => [super.isTimerVisible];
}

class SuccessSate extends AuthConfirmState {
  const SuccessSate(super.isTimerVisible);

  @override
  List<Object?> get props => [super.isTimerVisible];
}

class ErrorState extends AuthConfirmState {
  const ErrorState(super.isTimerVisible);

  @override
  List<Object?> get props => [super.isTimerVisible];
}

class ButtonEnableState extends AuthConfirmState {
  const ButtonEnableState(super.isTimerVisible, this.code);

  final String code;

  @override
  List<Object?> get props => [super.isTimerVisible, code];
}
